--
-- AdBluePallet
-- Specialization for AdBlue change
--
-- @author  	JoXXer / 
-- @version 	v1.0
-- @date  		16/11/11 / 
-- @history:	v1.0 - Initial version / rewritten and expanded by Blacky_BPG 26.02.2016
--

AdBluePallet = {};

function AdBluePallet.prerequisitesPresent(specializations)
    return true;
end;

function AdBluePallet:load(xmlFile)

	self.changeAdBluePallet = SpecializationUtil.callSpecializationsFunction("changeAdBluePallet");

	self.checkString = Utils.getNoNil(getXMLString(xmlFile, "vehicle.adBluePallet#checkString"), "changeAdBlue")
	local adBlueCapacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.adBluePallet#capacity"),80);
	self.adBlueCapacity = adBlueCapacity;
	self.adBlueDefaultCapacity = adBlueCapacity;
	local cans = getXMLString(xmlFile, "vehicle.adBluePallet#cans");
	self.cans = {};
	local canObject
	if cans ~= nil then
		canObject = Utils.indexToObject(self.components, cans);
		if canObject ~= nil then
			local num = getNumOfChildren(canObject);
			for i=0, num-1 do
				local child = getChildAt(canObject, i);
				if child ~= nil then
					table.insert(self.cans, child);
				end;
			end;
		end;
	end;
	self.canCount = table.getn(self.cans);
	self.lastCapacity = 0;

	self.adBluePalletVehicle = nil;
end;

function AdBluePallet:delete()
end;

function AdBluePallet:readStream(streamId, connection)
	self.adBlueCapacity = streamReadFloat32(streamId);
end;

function AdBluePallet:writeStream(streamId, connection)
	streamWriteFloat32(streamId, self.adBlueCapacity);
end;

function AdBluePallet:mouseEvent(posX, posY, isDown, isUp, button)
end;

function AdBluePallet:keyEvent(unicode, sym, modifier, isDown)
end;

function AdBluePallet:update(dt)
end;

function AdBluePallet:updateTick(dt)
	if self.lastCapacity ~= self.adBlueCapacity then
		if self.canCount ~= nil and self.canCount > 0 then
			local adBluePerCan = self.adBlueDefaultCapacity / self.canCount;
			local adBlueLeft = adBluePerCan;
			local capCheck = self.adBlueCapacity + adBluePerCan;
			for i=1, table.getn(self.cans) do
				if capCheck > adBlueLeft then
					if not getVisibility(self.cans[i]) then
						setVisibility(self.cans[i], true);
					end;
				else
					if getVisibility(self.cans[i]) then
						setVisibility(self.cans[i], false);
					end;
				end;
				adBlueLeft = adBlueLeft + adBluePerCan;
			end;
		end;
		self.lastCapacity = self.adBlueCapacity;
	end;
	local px, py, pz = getWorldTranslation(self.rootNode);
	local vehicle = g_currentMission.controlledVehicle;
	if vehicle ~= nil and vehicle.adBlueChange ~= nil and vehicle.adBlueChange.checkString == self.checkString then
		local vx, vy, vz = getWorldTranslation(vehicle.rootNode);
		local distance = Utils.vector2Length(px-vx, pz-vz);
		if distance < 10 then
			if vehicle.adBlueChange.adBlueInRange == nil or (vehicle.adBlueChange.adBlueInRange ~= nil and vehicle.adBlueChange.adBlueInRange ~= self and vehicle.neededAdBluePerChange <= self.adBlueCapacity and self.adBlueCapacity < vehicle.adBlueChange.adBlueInRange.adBlueCapacity) then
				vehicle.adBlueChange.adBlueInRange = self;
			end;
		else
			if vehicle.adBlueChange.adBlueInRange == self then
				vehicle.adBlueChange.adBlueInRange = nil;
			end;
		end;
	end;
end;

function AdBluePallet:draw()
end;

function AdBluePallet:changeAdBluePallet(vehicle, noEventSend)
	if vehicle ~= nil and vehicle.adBlueChange ~= nil then
		AdBlueChangeEventGlobal.sendEvent(self, vehicle, noEventSend);
		vehicle:changeAdBlue(self);
	end;
end;

function AdBluePallet:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	local valueInt = getXMLFloat(xmlFile, key.."#adBlueCapacity");
	if valueInt ~= nil then
		self.adBlueCapacity = valueInt;
	end;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function AdBluePallet:getSaveAttributesAndNodes(nodeIdent)
	local attributes = nil;
	attributes = 'adBlueCapacity="'.. self.adBlueCapacity .. '"';
	return attributes, nil;
end;


AdBlueChangeEventGlobal = {};
AdBlueChangeEventGlobal_mt = Class(AdBlueChangeEventGlobal, Event);

InitEventClass(AdBlueChangeEventGlobal, "AdBlueChangeEventGlobal");

function AdBlueChangeEventGlobal:emptyNew()
	local self = Event:new(AdBlueChangeEventGlobal_mt);
	self.className="AdBlueChangeEventGlobal";
	return self;
end;

function AdBlueChangeEventGlobal:new(vehicle, adBlueChangeVehicle)
	local self = AdBlueChangeEventGlobal:emptyNew();
	self.vehicle = vehicle;
	self.adBlueChangeVehicle = adBlueChangeVehicle;
	return self;
end;

function AdBlueChangeEventGlobal:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	local attacherId = streamReadInt32(streamId);
	self.adBlueChangeVehicle = networkGetObject(attacherId);
	self.vehicle = networkGetObject(id);
	self:run(connection);
end;

function AdBlueChangeEventGlobal:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteInt32(streamId, networkGetObjectId(self.adBlueChangeVehicle));
end;

function AdBlueChangeEventGlobal:run(connection)
	if self.vehicle ~= nil then
		self.vehicle:changeAdBluePallet(self.adBlueChangeVehicle, true);
		if not connection:getIsServer() then
			g_server:broadcastEvent(AdBlueChangeEventGlobal:new(self.vehicle, self.adBlueChangeVehicle), nil, connection, self.vehicle);
		end;
	end;
end;


function AdBlueChangeEventGlobal.sendEvent(vehicle, adBlueChangeVehicle, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(AdBlueChangeEventGlobal:new(vehicle, adBlueChangeVehicle), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(AdBlueChangeEventGlobal:new(vehicle, adBlueChangeVehicle));
		end;
	end;
end;
